@@ -1,10 +1,11 @@
 from __future__ import annotations
 
-from typing import Union
 from pathlib import Path
-from .range import Singular, MRange
 from .tables import SimulationEntry, InstrEntry
 
+def mcpl_parameter_split(s: str) -> tuple[str, str]:
+    k, v = s.split(':', 1)
+    return k, v
 
 def make_splitrun_parser():
     from argparse import ArgumentParser
@@ -30,9 +31,21 @@ def make_splitrun_parser():
        help='Maximum number of particles to simulate during first instrument simulations')
     aa('--dryrun', action='store_true', default=False,
        help='Do not run any simulations, just print the commands')
+    aa('--parallel', action='store_true', default=False,
+       help='Use MPI multi-process parallelism (primary instrument only at the moment)')
+    aa('--gpu', action='store_true', default=False,
+       help='Use GPU OpenACC parallelism (primary instrument only at the moment)')
+    aa('--process-count',  nargs=1, type=int, default=0,
+       help='MPI process count, 0 == System Default')
     # splitrun controlling parameters
     aa('--split-at', nargs=1, type=str, default=['mcpl_split'],
        help='Component at which to split -- DEFAULT: mcpl_split')
+    aa('--mcpl-output-component', nargs=1, type=str, default=None,
+       help='Inserted MCPL file producing component, MCPL_output(.comp) if not provided')
+    aa('--mcpl-input-component', nargs=1, type=str, default=None,
+       help='Inserted MCPL file consuming component, MCPL_input(.comp) if not provided')
+    aa('--mcpl-input-parameters', nargs='+', type=mcpl_parameter_split, metavar='key:value')
+    aa('--mcpl-output-parameters', nargs='+', type=mcpl_parameter_split, metavar='key:value')
     aa('-P', action='append', default=[], help='Cache parameter matching precision')
 
     # Other McCode runtime arguments exist, but are likely not used during a scan:
@@ -71,31 +84,6 @@ def regular_mccode_runtime_dict(args: dict) -> dict:
     return t
 
 
-def mccode_runtime_dict_to_args_list(args: dict) -> list[str]:
-    """Convert a dictionary of McCode runtime arguments to a string.
-
-    :parameter args: A dictionary of McCode runtime arguments.
-    :return: A list of arguments suitable for use in a command line call to a McCode compiled instrument.
-    """
-    # convert to a standardized string:
-    out = []
-    if 'seed' in args and args['seed'] is not None:
-        out.append(f'--seed={args["seed"]}')
-    if 'ncount' in args and args['ncount'] is not None:
-        out.append(f'--ncount={args["ncount"]}')
-    if 'dir' in args and args['dir'] is not None:
-        out.append(f'--dir={args["dir"]}')
-    if 'trace' in args and args['trace']:
-        out.append('--trace')
-    if 'gravitation' in args and args['gravitation']:
-        out.append('--gravitation')
-    if 'bufsiz' in args and args['bufsiz'] is not None:
-        out.append(f'--bufsiz={args["bufsiz"]}')
-    if 'format' in args and args['format'] is not None:
-        out.append(f'--format={args["format"]}')
-    return out
-
-
 def parse_splitrun_precision(unparsed: list[str]) -> dict[str, float]:
     precision = {}
     for p in unparsed:
@@ -106,43 +94,34 @@ def parse_splitrun_precision(unparsed: list[str]) -> dict[str, float]:
     return precision
 
 
-def sort_args(args: list[str]) -> list[str]:
-    """Take the list of arguments and sort them into the correct order for splitrun"""
-    # TODO this is a bit of a hack, but it works for now
-    first, last = [], []
-    k = 0
-    while k < len(args):
-        if args[k].startswith('-'):
-            first.append(args[k])
-            k += 1
-            if '=' not in first[-1] and k < len(args) and not args[k].startswith('-') and '=' not in args[k]:
-                first.append(args[k])
-                k += 1
-        else:
-            last.append(args[k])
-            k += 1
-    return first + last
-
-
-def parse_splitrun():
+def parse_splitrun(parser):
     from .range import parse_scan_parameters
+    from mccode_antlr.run.runner import sort_args
     import sys
     sys.argv[1:] = sort_args(sys.argv[1:])
 
-    args = make_splitrun_parser().parse_args()
+    args = parser.parse_args()
+    if args.mcpl_input_parameters is not None:
+        args.mcpl_input_parameters = dict(args.mcpl_input_parameters)
+    if args.mcpl_output_parameters is not None:
+        args.mcpl_output_parameters = dict(args.mcpl_output_parameters)
     parameters = parse_scan_parameters(args.parameters)
     precision = parse_splitrun_precision(args.P)
     return args, parameters, precision
 
 
 def entrypoint():
-    args, parameters, precision = parse_splitrun()
+    args, parameters, precision = parse_splitrun(make_splitrun_parser())
     splitrun_from_file(args, parameters, precision)
 
 
 def splitrun_from_file(args, parameters, precision):
     from .instr import load_instr
     instr = load_instr(args.instrument[0])
+    splitrun_args(instr, parameters, precision, args)
+
+
+def splitrun_args(instr, parameters, precision, args, **kwargs):
     splitrun(instr, parameters, precision, split_at=args.split_at[0], grid=args.mesh,
              seed=args.seed[0] if args.seed is not None else None,
              ncount=args.ncount[0] if args.ncount is not None else None,
@@ -153,7 +132,15 @@ def splitrun_from_file(args, parameters, precision):
              format=args.format[0] if args.format is not None else None,
              minimum_particle_count=args.nmin[0] if args.nmin is not None else None,
              maximum_particle_count=args.nmax[0] if args.nmax is not None else None,
-             dry_run=args.dryrun
+             dry_run=args.dryrun,
+             parallel=args.parallel,
+             gpu=args.gpu,
+             process_count=args.process_count,
+             mcpl_output_component=args.mcpl_output_component[0] if args.mcpl_output_component is not None else None,
+             mcpl_output_parameters=args.mcpl_output_parameters,
+             mcpl_input_component=args.mcpl_input_component[0] if args.mcpl_input_component is not None else None,
+             mcpl_input_parameters=args.mcpl_input_parameters,
+             **kwargs
              )
 
 
@@ -161,9 +148,14 @@ def splitrun(instr, parameters, precision: dict[str, float], split_at=None, grid
              minimum_particle_count=None,
              maximum_particle_count=None,
              dry_run=False,
+             parallel=False, gpu=False, process_count=0,
              callback=None, callback_arguments: dict[str, str] | None = None,
+             output_split_instrs=True,
+             mcpl_output_component=None, mcpl_output_parameters: dict[str, str] | None = None,
+             mcpl_input_component=None, mcpl_input_parameters: dict[str, str] | None = None,
              **runtime_arguments):
     from zenlog import log
+    from mccode_antlr.common import ComponentParameter, Expr
     from .energy import get_energy_parameter_names
     if split_at is None:
         split_at = 'mcpl_split'
@@ -171,7 +163,21 @@ def splitrun(instr, parameters, precision: dict[str, float], split_at=None, grid
     if not instr.has_component_named(split_at):
         log.error(f'The specified split-at component, {split_at}, does not exist in the instrument file')
     # splitting defines an instrument parameter in both returned instrument, 'mcpl_filename'.
-    pre, post = instr.mcpl_split(split_at, remove_unused_parameters=True)
+    if mcpl_output_parameters is not None:
+        mcpl_output_parameters = tuple(ComponentParameter(k, Expr.parse(v)) for k, v in mcpl_output_parameters.items())
+    if mcpl_input_parameters is not None:
+        mcpl_input_parameters = tuple(ComponentParameter(k, Expr.parse(v)) for k, v in mcpl_input_parameters.items())
+    pre, post = instr.mcpl_split(split_at,
+                                 output_component=mcpl_output_component,
+                                 output_parameters=mcpl_output_parameters,
+                                 input_component=mcpl_input_component,
+                                 input_parameters=mcpl_input_parameters,
+                                 remove_unused_parameters=True
+                                 )
+    if output_split_instrs:
+        for p in (pre, post):
+            with open(f'{p.name}.instr', 'w') as f:
+                p.to_file(f)
     # ... reduce the parameters to those that are relevant to the two instruments.
     pre_parameters = {k: v for k, v in parameters.items() if pre.has_parameter(k)}
     post_parameters = {k: v for k, v in parameters.items() if post.has_parameter(k)}
@@ -185,24 +191,25 @@ def splitrun(instr, parameters, precision: dict[str, float], split_at=None, grid
     pre_entry = splitrun_pre(pre, pre_parameters, grid, precision, **runtime_arguments,
                              minimum_particle_count=minimum_particle_count,
                              maximum_particle_count=maximum_particle_count,
-                             dry_run=dry_run)
+                             dry_run=dry_run, parallel=parallel, gpu=gpu, process_count=process_count)
 
     splitrun_combined(pre_entry, pre, post, pre_parameters, post_parameters, grid, precision,
-                      dry_run=dry_run, callback=callback, callback_arguments=callback_arguments, **runtime_arguments)
+                      dry_run=dry_run, parallel=parallel, gpu=gpu, process_count=process_count,
+                      callback=callback, callback_arguments=callback_arguments, **runtime_arguments)
 
 
 def splitrun_pre(instr, parameters, grid, precision: dict[str, float],
                  minimum_particle_count=None, maximum_particle_count=None, dry_run=False,
+                 parallel=False, gpu=False, process_count=0,
                  **runtime_arguments):
 
     from functools import partial
     from .cache import cache_instr
     from .energy import energy_to_chopper_translator
     from .range import parameters_to_scan
-
     # check if this instr is already represented in the module's cache database
     # if not, it is compiled and added to the cache with (hopefully sensible) defaults specified
-    entry = cache_instr(instr)
+    entry = cache_instr(instr, mpi=parallel, acc=gpu)
     # get the function with converts energy parameters to chopper parameters:
     translate = energy_to_chopper_translator(instr.name)
     # determine the scan in the user-defined parameters!
@@ -211,7 +218,7 @@ def splitrun_pre(instr, parameters, grid, precision: dict[str, float],
     sit_kw = {'seed': args.get('seed'), 'ncount': args.get('ncount'), 'gravitation': args.get('gravitation', False)}
 
     step = partial(_pre_step, instr, entry, names, precision, translate, sit_kw, minimum_particle_count,
-                   maximum_particle_count, dry_run)
+                   maximum_particle_count, dry_run, process_count)
 
     # this does not work due to the sqlite database being locked by the parallel processes
     # from joblib import Parallel, delayed
@@ -225,7 +232,7 @@ def splitrun_pre(instr, parameters, grid, precision: dict[str, float],
     return entry
 
 
-def _pre_step(instr, entry, names, precision, translate, kw, min_pc, max_pc, dry_run, values):
+def _pre_step(instr, entry, names, precision, translate, kw, min_pc, max_pc, dry_run, process_count, values):
     """The per-step function for the primary instrument simulation. Broken out for parallelization"""
     from .instr import collect_parameter_dict
     from .cache import cache_has_simulation, cache_simulation, cache_get_simulation
@@ -235,13 +242,15 @@ def _pre_step(instr, entry, names, precision, translate, kw, min_pc, max_pc, dry
         sim.output_path = do_primary_simulation(sim, entry, nv, kw,
                                                 minimum_particle_count=min_pc,
                                                 maximum_particle_count=max_pc,
-                                                dry_run=dry_run)
+                                                dry_run=dry_run,
+                                                process_count=process_count)
         cache_simulation(entry, sim)
     return cache_get_simulation(entry, sim)
 
 
 def splitrun_combined(pre_entry, pre, post, pre_parameters, post_parameters, grid, precision: dict[str, float],
                       summary=True, dry_run=False, callback=None, callback_arguments: dict[str, str] | None = None,
+                      parallel=False, gpu=False, process_count=0,
                       **runtime_arguments):
     from pathlib import Path
     from .cache import cache_instr, cache_get_simulation
@@ -250,7 +259,7 @@ def splitrun_combined(pre_entry, pre, post, pre_parameters, post_parameters, gri
     from .instr import collect_parameter_dict
     from .tables import best_simulation_entry_match
     from .emulate import mccode_sim_io, mccode_dat_io, mccode_dat_line
-    instr_entry = cache_instr(post)
+    instr_entry = cache_instr(post, mpi=parallel, acc=gpu)
     args = regular_mccode_runtime_dict(runtime_arguments)
     sit_kw = {'seed': args.get('seed'), 'ncount': args.get('ncount'), 'gravitation': args.get('gravitation', False)}
     # recombine the parameters to ensure the 'correct' scan is performed
@@ -291,7 +300,7 @@ def splitrun_combined(pre_entry, pre, post, pre_parameters, post_parameters, gri
         # TODO Use the following line instead of the one after it when McCode is fixed to use zero-padded folder names
         # # runtime_arguments['dir'] = args["dir"].joinpath(str(number).zfill(n_zeros))
         runtime_arguments['dir'] = args['dir'].joinpath(str(number))
-        do_secondary_simulation(sim_entry, instr_entry, secondary_pars, runtime_arguments, dry_run=dry_run)
+        do_secondary_simulation(sim_entry, instr_entry, secondary_pars, runtime_arguments, dry_run=dry_run, process_count=process_count)
         if summary and not dry_run:
             # the data file has *all* **scanned** parameters recorded for each step:
             detectors, line = mccode_dat_line(runtime_arguments['dir'], {k: v for k,v in zip(names, values)})
@@ -315,7 +324,8 @@ def splitrun_combined(pre_entry, pre, post, pre_parameters, post_parameters, gri
 
 
 def _args_pars_mcpl(args: dict, params: dict, mcpl_filename) -> str:
-    # Combine the arguments, parameters, and mcpl filename into a single command-arguments string:
+    """Combine the arguments, parameters, and mcpl filename into a single command-arguments string:"""
+    from mccode_antlr.run.runner import mccode_runtime_dict_to_args_list
     first = ' '.join(mccode_runtime_dict_to_args_list(args))
     second = ' '.join([f'{k}={v}' for k, v in params.items()])
     third = f'mcpl_filename={mcpl_filename}'
@@ -336,7 +346,8 @@ def do_primary_simulation(sit: SimulationEntry,
                           args: dict,
                           minimum_particle_count: int | None = None,
                           maximum_particle_count: int | None = None,
-                          dry_run: bool = False
+                          dry_run: bool = False,
+                          process_count: int = 0,
                           ):
     from zenlog import log
     from pathlib import Path
@@ -344,10 +355,11 @@ def do_primary_simulation(sit: SimulationEntry,
     from mccode_antlr.compiler.c import run_compiled_instrument, CBinaryTarget
     from .cache import directory_under_module_data_path
     # create a directory for this simulation based on the uuid generated for the simulation entry
-    work_dir = directory_under_module_data_path('sim', prefix=f'p_{instr_file_entry.id}_')
+    work_dir = directory_under_module_data_path('sim', prefix=f'{Path(instr_file_entry.binary_path).parent.stem}_')
 
     binary_at = Path(instr_file_entry.binary_path)
-    target = CBinaryTarget(mpi=False, acc=False, count=1, nexus=False)
+    # process_count == 0 --> system MPI default process count (# physical cores, typically)
+    target = CBinaryTarget(mpi=instr_file_entry.mpi, acc=instr_file_entry.acc, count=process_count, nexus=False)
 
     # ensure the primary spectrometer uses our output directory
     args_dict = {k: v for k, v in args.items() if k != 'dir'}
@@ -363,6 +375,13 @@ def do_primary_simulation(sit: SimulationEntry,
         log.info('Expected mcpl_filename parameter in primary simulation, using default')
         mcpl_filename = f'{sit.id}.mcpl'
         sit.parameter_values['mcpl_filename'] = Value.str(mcpl_filename)
+
+    # strip the extension from the filename passed into the runner:
+    if mcpl_filename.endswith('.gz'):
+        mcpl_filename = mcpl_filename[:-3]
+    if mcpl_filename.endswith('.mcpl'):
+        mcpl_filename = mcpl_filename[:-5]
+
     mcpl_filepath = work_dir.joinpath(mcpl_filename)
     runner = partial(run_compiled_instrument, binary_at, target, capture=False, dry_run=dry_run)
     if dry_run or args.get('ncount') is None:
@@ -387,7 +406,7 @@ def repeat_simulation_until(count, runner, args: dict, parameters, work_dir: Pat
     from functools import partial
     from zenlog import log
     from .emulate import combine_mccode_dats_in_directories, combine_mccode_sims_in_directories
-    from .mcpl import mcpl_particle_count, mcpl_merge_files
+    from .mcpl import mcpl_particle_count, mcpl_merge_files, mcpl_rename_file
     goal, latest_result, one_trillion = count, -1, 1_000_000_000_000
     # avoid looping for too long by limiting the minimum number of particles to simulate
     minimum_particle_count = _clamp(1, one_trillion, minimum_particle_count or count)
@@ -415,21 +434,24 @@ def repeat_simulation_until(count, runner, args: dict, parameters, work_dir: Pat
             args['seed'] = random.randint(1, 2 ** 32 - 1)
 
         outputs.append(work_dir.joinpath(f'{len(files)}'))
-        files.append(work_dir.joinpath(f'part_{len(files)}.mcpl'))
+        files.append(work_dir.joinpath(f'part_{len(files)}'))  # appending the extension here breaks MCPL+MPI?
         args['dir'] = outputs[-1]
         # adjust our guess for how many particles to simulate : how many we need divided by the last transmission
         args['ncount'] = clamp(((goal - sum(counts)) * args['ncount']) // counts[-1] if len(counts) else goal)
-        runner(_args_pars_mcpl(args, parameters, files[-1]))
-        counts.append(mcpl_particle_count(files[-1]))
+        # recycle the intended-output mcpl filename to avoid breaking mcpl file-merging
+        runner(_args_pars_mcpl(args, parameters, mcpl_filepath))
+        counts.append(mcpl_particle_count(mcpl_filepath))
+        # rename the outputfile to this run's filename
+        files[-1] = mcpl_rename_file(mcpl_filepath, files[-1])
 
     # now we need to concatenate the mcpl files, and combine output (.dat and .sim) files
-    mcpl_merge_files(files, str(mcpl_filepath))
+    mcpl_merge_files(files, mcpl_filepath)
     combine_mccode_dats_in_directories(outputs, work_dir)
     combine_mccode_sims_in_directories(outputs, work_dir)
 
 
-def do_secondary_simulation(p_sit: SimulationEntry, entry: InstrEntry, pars: dict, args: dict,
-                            dry_run: bool = False):
+def do_secondary_simulation(p_sit: SimulationEntry, entry: InstrEntry, pars: dict, args: dict, dry_run: bool = False,
+                            process_count: int = 0):
     from zenlog import log
     from pathlib import Path
     from shutil import copy
@@ -447,7 +469,7 @@ def do_secondary_simulation(p_sit: SimulationEntry, entry: InstrEntry, pars: dic
 
     mcpl_path = mcpl_real_filename(Path(p_sit.output_path).joinpath(mcpl_filename))
     executable = Path(entry.binary_path)
-    target = CBinaryTarget(mpi=False, acc=False, count=1, nexus=False)
+    target = CBinaryTarget(mpi=entry.mpi, acc=entry.acc, count=process_count, nexus=False)
     run_compiled_instrument(executable, target, _args_pars_mcpl(args, pars, mcpl_path), capture=False, dry_run=dry_run)
 
     if not dry_run:
